@@ -70,7 +70,7 @@ static uint32_t FragmentDataParser(Flow *f, void *dcerpcudp_state,
     stub_len = (sstate->dcerpc.fraglenleft < input_len) ? sstate->dcerpc.fraglenleft : input_len;
 
     if (stub_len == 0) {
-	SCReturnUInt(0);
+        SCReturnUInt(0);
     }
     /* if the frag is the the first frag irrespective of it being a part of
      * a multi frag PDU or not, it indicates the previous PDU's stub would
@@ -194,7 +194,7 @@ static int DCERPCUDPParseHeader(Flow *f, void *dcerpcudp_state,
 					sstate->dcerpc.dcerpchdrudp.if_vers = *(p + 60);
 					sstate->dcerpc.dcerpchdrudp.if_vers |= *(p + 61) << 8;
 					sstate->dcerpc.dcerpchdrudp.if_vers |= *(p + 62) << 16;
-					sstate->dcerpc.dcerpchdrudp.if_vers |= *(p + 63) >> 24;
+					sstate->dcerpc.dcerpchdrudp.if_vers |= *(p + 63) << 24;
 					sstate->dcerpc.dcerpchdrudp.seqnum = *(p + 64);
 					sstate->dcerpc.dcerpchdrudp.seqnum |= *(p + 65) << 8;
 					sstate->dcerpc.dcerpchdrudp.seqnum |= *(p + 66) << 16;
@@ -630,14 +630,15 @@ static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
 	while (sstate->bytesprocessed < DCERPC_UDP_HDR_LEN && input_len) {
 		hdrretval = DCERPCUDPParseHeader(f, dcerpc_state, pstate, input,
 				input_len, output);
-		if(hdrretval == -1) {
+		if (hdrretval == -1 || hdrretval > (int32_t)input_len) {
 			sstate->bytesprocessed = 0;
 			SCReturnInt(hdrretval);
 		} else {
-			parsed += retval;
-			input_len -= retval;
+			parsed += hdrretval;
+			input_len -= hdrretval;
 		}
 	}
+
 #if 0
 	printf("Done with DCERPCUDPParseHeader bytesprocessed %u/%u left %u\n",
 			sstate->bytesprocessed, sstate->dcerpc.dcerpchdrudp.fraglen, input_len);
@@ -659,7 +660,7 @@ static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
 			&& input_len) {
 		retval = FragmentDataParser(f, dcerpc_state, pstate, input + parsed,
 				input_len, output);
-		if (retval) {
+		if (retval || retval > input_len) {
 			parsed += retval;
 			input_len -= retval;
 		} else if (input_len) {
@@ -717,12 +718,14 @@ static void DCERPCUDPStateFree(void *s) {
 }
 
 void RegisterDCERPCUDPParsers(void) {
-    /** DCERPC */
-    AlpProtoAdd(&alp_proto_ctx, IPPROTO_UDP, ALPROTO_DCERPC_UDP, "|04 00|", 2, 0, STREAM_TOSERVER);
+    char *proto_name = "dcerpcudp";
 
-	AppLayerRegisterProto("dcerpcudp", ALPROTO_DCERPC_UDP, STREAM_TOSERVER,
+    /** DCERPC */
+    AlpProtoAdd(&alp_proto_ctx, proto_name, IPPROTO_UDP, ALPROTO_DCERPC_UDP, "|04 00|", 2, 0, STREAM_TOSERVER);
+
+	AppLayerRegisterProto(proto_name, ALPROTO_DCERPC_UDP, STREAM_TOSERVER,
 			DCERPCUDPParse);
-	AppLayerRegisterProto("dcerpcudp", ALPROTO_DCERPC_UDP, STREAM_TOCLIENT,
+	AppLayerRegisterProto(proto_name, ALPROTO_DCERPC_UDP, STREAM_TOCLIENT,
 			DCERPCUDPParse);
 	AppLayerRegisterStateFuncs(ALPROTO_DCERPC_UDP, DCERPCUDPStateAlloc,
 			DCERPCUDPStateFree);
