@@ -47,24 +47,26 @@
 
 #define HTP_FLAG_STATE_OPEN         0x0001    /**< Flag to indicate that HTTP
                                              connection is open */
-#define HTP_FLAG_STATE_CLOSED       0x0002    /**< Flag to indicate that HTTP
+#define HTP_FLAG_STATE_CLOSED_TS    0x0002    /**< Flag to indicate that HTTP
                                              connection is closed */
-#define HTP_FLAG_STATE_DATA         0x0004    /**< Flag to indicate that HTTP
+#define HTP_FLAG_STATE_CLOSED_TC    0x0004    /**< Flag to indicate that HTTP
+                                             connection is closed */
+#define HTP_FLAG_STATE_DATA         0x0008    /**< Flag to indicate that HTTP
                                              connection needs more data */
-#define HTP_FLAG_STATE_ERROR        0x0008    /**< Flag to indicate that an error
+#define HTP_FLAG_STATE_ERROR        0x0010    /**< Flag to indicate that an error
                                              has been occured on HTTP
                                              connection */
-#define HTP_FLAG_NEW_BODY_SET       0x0010    /**< Flag to indicate that HTTP
+#define HTP_FLAG_NEW_BODY_SET       0x0020    /**< Flag to indicate that HTTP
                                              has parsed a new body (for
                                              pcre) */
-#define HTP_FLAG_STORE_FILES_TS     0x0020
-#define HTP_FLAG_STORE_FILES_TC     0x0040
-#define HTP_FLAG_STORE_FILES_TX_TS  0x0080
-#define HTP_FLAG_STORE_FILES_TX_TC  0x0100
+#define HTP_FLAG_STORE_FILES_TS     0x0040
+#define HTP_FLAG_STORE_FILES_TC     0x0080
+#define HTP_FLAG_STORE_FILES_TX_TS  0x0100
+#define HTP_FLAG_STORE_FILES_TX_TC  0x0200
 /** flag the state that a new file has been set in this tx */
-#define HTP_FLAG_NEW_FILE_TX_TS     0x0200
+#define HTP_FLAG_NEW_FILE_TX_TS     0x0400
 /** flag the state that a new file has been set in this tx */
-#define HTP_FLAG_NEW_FILE_TX_TC     0x0400
+#define HTP_FLAG_NEW_FILE_TX_TC     0x0800
 
 enum {
     HTP_BODY_NONE = 0,                  /**< Flag to indicate the current
@@ -77,11 +79,13 @@ enum {
 
 enum {
     HTP_BODY_REQUEST_NONE = 0,
-    HTP_BODY_REQUEST_MULTIPART,
+    HTP_BODY_REQUEST_MULTIPART, /* POST, MP */
+    HTP_BODY_REQUEST_POST,      /* POST, no MP */
     HTP_BODY_REQUEST_PUT,
 };
 
 enum {
+    /* libhtp errors/warnings */
     HTTP_DECODER_EVENT_UNKNOWN_ERROR,
     HTTP_DECODER_EVENT_GZIP_DECOMPRESSION_FAILED,
     HTTP_DECODER_EVENT_REQUEST_FIELD_MISSING_COLON,
@@ -104,6 +108,11 @@ enum {
     HTTP_DECODER_EVENT_INVALID_RESPONSE_FIELD_FOLDING,
     HTTP_DECODER_EVENT_REQUEST_FIELD_TOO_LONG,
     HTTP_DECODER_EVENT_RESPONSE_FIELD_TOO_LONG,
+
+    /* suricata errors/warnings */
+    HTTP_DECODER_EVENT_MULTIPART_GENERIC_ERROR,
+    HTTP_DECODER_EVENT_MULTIPART_NO_FILEDATA,
+    HTTP_DECODER_EVENT_MULTIPART_INVALID_HEADER,
 };
 
 #define HTP_PCRE_NONE           0x00    /**< No pcre executed yet */
@@ -114,38 +123,37 @@ enum {
                                              matched on some rule */
 
 /** Struct used to hold chunks of a body on a request */
-typedef struct HtpBodyChunk_ {
+struct HtpBodyChunk_ {
     uint8_t *data;              /**< Pointer to the data of the chunk */
-    uint32_t len;               /**< Length of the chunk */
-    uint32_t id;                /**< number of chunk of the current body */
     struct HtpBodyChunk_ *next; /**< Pointer to the next chunk */
     uint64_t stream_offset;
-} HtpBodyChunk;
+    uint32_t len;               /**< Length of the chunk */
+} __attribute__((__packed__));
+typedef struct HtpBodyChunk_ HtpBodyChunk;
 
 /** Struct used to hold all the chunks of a body on a request */
 typedef struct HtpBody_ {
     HtpBodyChunk *first; /**< Pointer to the first chunk */
     HtpBodyChunk *last;  /**< Pointer to the last chunk */
-    uint32_t nchunks;    /**< Number of chunks in the current operation */
-    uint8_t type;
 
     /* Holds the length of the htp request body */
     uint64_t content_len;
     /* Holds the length of the htp request body seen so far */
     uint64_t content_len_so_far;
-
+    /* parser tracker */
     uint64_t body_parsed;
-
-    /* pahole: padding: 3 */
+    /* inspection tracker */
+    uint64_t body_inspected;
 } HtpBody;
 
-#define HTP_BODY_COMPLETE       0x01    /**< body is complete or limit is reached,
+#define HTP_REQ_BODY_COMPLETE   0x01    /**< body is complete or limit is reached,
                                              either way, this is it. */
-#define HTP_CONTENTTYPE_SET     0x02    /**< We have the content type */
-#define HTP_BOUNDARY_SET        0x04    /**< We have a boundary string */
-#define HTP_BOUNDARY_OPEN       0x08    /**< We have a boundary string */
-#define HTP_FILENAME_SET        0x10    /**< filename is registered in the flow */
-#define HTP_DONTSTORE           0x20    /**< not storing this file */
+#define HTP_RES_BODY_COMPLETE   0x02
+#define HTP_CONTENTTYPE_SET     0x04    /**< We have the content type */
+#define HTP_BOUNDARY_SET        0x08    /**< We have a boundary string */
+#define HTP_BOUNDARY_OPEN       0x10    /**< We have a boundary string */
+#define HTP_FILENAME_SET        0x20    /**< filename is registered in the flow */
+#define HTP_DONTSTORE           0x40    /**< not storing this file */
 
 #define HTP_TX_HAS_FILE             0x01
 #define HTP_TX_HAS_FILENAME         0x02    /**< filename is known at this time */
@@ -170,9 +178,14 @@ typedef struct HtpTxUserData_ {
     uint8_t *boundary;
     uint8_t boundary_len;
 
-    uint8_t flags;
+    uint8_t tsflags;
+    uint8_t tcflags;
 
     int16_t operation;
+
+    uint8_t request_body_type;
+    uint8_t response_body_type;
+
 } HtpTxUserData;
 
 typedef struct HtpState_ {
@@ -190,6 +203,16 @@ typedef struct HtpState_ {
     FileContainer *files_tc;
 } HtpState;
 
+/** part of the engine needs the request body (e.g. http_client_body keyword) */
+extern uint8_t need_htp_request_body;
+/** part of the engine needs the request body multipart header (e.g. filename
+ *  and / or fileext keywords) */
+extern uint8_t need_htp_request_multipart_hdr;
+/** part of the engine needs the request file (e.g. log-file module) */
+extern uint8_t need_htp_request_file;
+/** part of the engine needs the request body (e.g. file_data keyword) */
+extern uint8_t need_htp_response_body;
+
 void RegisterHTPParsers(void);
 void HTPParserRegisterTests(void);
 void HTPAtExitPrintStats(void);
