@@ -41,24 +41,13 @@
 #include "detect-engine-mpm.h"
 #include "detect-parse.h"
 #include "detect-engine-state.h"
-#include "detect-pcre.h"
-#include "detect-isdataat.h"
-#include "detect-bytetest.h"
-#include "detect-bytejump.h"
-#include "detect-byte-extract.h"
+#include "detect-engine-content-inspection.h"
 
 #include "flow-util.h"
-#include "util-spm.h"
 #include "util-debug.h"
 #include "util-print.h"
 #include "flow.h"
-#include "detect-flow.h"
-#include "flow-var.h"
-#include "threads.h"
-#include "flow-alert-sid.h"
 
-#include "stream-tcp.h"
-#include "stream.h"
 #include "app-layer-parser.h"
 
 #include "util-unittest.h"
@@ -67,320 +56,8 @@
 #include "app-layer-htp.h"
 #include "app-layer-protos.h"
 
-/**
- * \brief Run the actual payload match function for http response body.
- *
- *        For accounting the last match in relative matching the
- *        det_ctx->payload_offset int is used.
- *
- * \param de_ctx      Detection engine context.
- * \param det_ctx     Detection engine thread context.
- * \param s           Signature to inspect.
- * \param sm          SigMatch to inspect.
- * \param payload     Ptr to the response body to inspect.
- * \param payload_len Length of the response body.
- *
- * \retval 0 no match.
- * \retval 1 match.
- */
-static int DoInspectHttpServerBody(DetectEngineCtx *de_ctx,
-                                   DetectEngineThreadCtx *det_ctx,
-                                   Signature *s, SigMatch *sm,
-                                   uint8_t *payload, uint32_t payload_len)
-{
-    SCEnter();
-
-    det_ctx->inspection_recursion_counter++;
-
-    if (det_ctx->inspection_recursion_counter == de_ctx->inspection_recursion_limit) {
-        det_ctx->discontinue_matching = 1;
-        SCReturnInt(0);
-    }
-
-    if (sm == NULL) {
-        SCReturnInt(0);
-    }
-
-    if (sm->type == DETECT_AL_HTTP_SERVER_BODY) {
-        if (payload_len == 0) {
-            SCReturnInt(0);
-        }
-
-        DetectContentData *cd = (DetectContentData *)sm->ctx;
-        SCLogDebug("inspecting content %"PRIu32" payload_len %"PRIu32, cd->id, payload_len);
-
-        //if (cd->flags & DETECT_CONTENT_HSBD_MPM && !(cd->flags & DETECT_CONTENT_NEGATED))
-        //    goto match;
-
-        /* rule parsers should take care of this */
-#ifdef DEBUG
-        BUG_ON(cd->depth != 0 && cd->depth <= cd->offset);
-#endif
-
-        /* search for our pattern, checking the matches recursively.
-         * if we match we look for the next SigMatch as well */
-        uint8_t *found = NULL;
-        uint32_t offset = 0;
-        uint32_t depth = payload_len;
-        uint32_t prev_offset = 0; /**< used in recursive searching */
-        uint32_t prev_payload_offset = det_ctx->payload_offset;
-
-        do {
-            if (cd->flags & DETECT_CONTENT_DISTANCE ||
-                cd->flags & DETECT_CONTENT_WITHIN) {
-                SCLogDebug("prev_payload_offset %"PRIu32, prev_payload_offset);
-
-                offset = prev_payload_offset;
-                depth = payload_len;
-
-                if (cd->flags & DETECT_CONTENT_DISTANCE) {
-                    if (cd->distance < 0 && (uint32_t)(abs(cd->distance)) > offset)
-                        offset = 0;
-                    else
-                        offset += cd->distance;
-                }
-
-                if (cd->flags & DETECT_CONTENT_WITHIN) {
-                    if ((int32_t)depth > (int32_t)(prev_payload_offset + cd->within + cd->distance)) {
-                        depth = prev_payload_offset + cd->within + cd->distance;
-                    }
-                }
-
-                if (cd->depth != 0) {
-                    if ((cd->depth + prev_payload_offset) < depth) {
-                        depth = prev_payload_offset + cd->depth;
-                    }
-                }
-
-                if (cd->offset > offset) {
-                    offset = cd->offset;
-                }
-            } else { /* implied no relative matches */
-                /* set depth */
-                if (cd->depth != 0) {
-                    depth = cd->depth;
-                }
-
-                /* set offset */
-                offset = cd->offset;
-                prev_payload_offset = 0;
-            }
-
-            /* update offset with prev_offset if we're searching for
-             * matches after the first occurence. */
-            if (prev_offset != 0)
-                offset = prev_offset;
-
-            if (depth > payload_len)
-                depth = payload_len;
-
-            /* if offset is bigger than depth we can never match on a pattern.
-             * We can however, "match" on a negated pattern. */
-            if (offset > depth || depth == 0) {
-                if (cd->flags & DETECT_CONTENT_NEGATED) {
-                    goto match;
-                } else {
-                    SCReturnInt(0);
-                }
-            }
-
-            uint8_t *spayload = payload + offset;
-            uint32_t spayload_len = depth - offset;
-            uint32_t match_offset = 0;
-#ifdef DEBUG
-            BUG_ON(spayload_len > payload_len);
-#endif
-
-            /* do the actual search with boyer moore precooked ctx */
-            if (cd->flags & DETECT_CONTENT_NOCASE) {
-                found = BoyerMooreNocase(cd->content, cd->content_len,
-                                         spayload, spayload_len,
-                                         cd->bm_ctx->bmGs, cd->bm_ctx->bmBc);
-            } else {
-                found = BoyerMoore(cd->content, cd->content_len,
-                                   spayload, spayload_len,
-                                   cd->bm_ctx->bmGs, cd->bm_ctx->bmBc);
-            }
-
-            /* next we evaluate the result in combination with the
-             * negation flag. */
-            if (found == NULL && !(cd->flags & DETECT_CONTENT_NEGATED)) {
-                SCReturnInt(0);
-            } else if (found == NULL && cd->flags & DETECT_CONTENT_NEGATED) {
-                goto match;
-            } else if (found != NULL && cd->flags & DETECT_CONTENT_NEGATED) {
-                det_ctx->discontinue_matching = 1;
-                SCReturnInt(0);
-            } else {
-                match_offset = (uint32_t)((found - payload) + cd->content_len);
-                det_ctx->payload_offset = match_offset;
-
-                if (!(cd->flags & DETECT_CONTENT_RELATIVE_NEXT)) {
-                    SCLogDebug("no relative match coming up, so this is a match");
-                    goto match;
-                }
-
-                /* bail out if we have no next match. Technically this is an
-                 * error, as the current cd has the DETECT_CONTENT_RELATIVE_NEXT
-                 * flag set. */
-                if (sm->next == NULL) {
-                    SCReturnInt(0);
-                }
-
-                /* see if the next payload keywords match. If not, we will
-                 * search for another occurence of this http server body content
-                 * and see if the others match then until we run out of matches */
-                int r = DoInspectHttpServerBody(de_ctx, det_ctx, s, sm->next,
-                                                payload, payload_len);
-                if (r == 1) {
-                    SCReturnInt(1);
-                }
-
-                if (det_ctx->discontinue_matching)
-                    SCReturnInt(0);
-
-                /* set the previous match offset to the start of this match + 1 */
-                prev_offset = (match_offset - (cd->content_len - 1));
-                SCLogDebug("trying to see if there is another match after "
-                           "prev_offset %"PRIu32, prev_offset);
-            }
-
-        } while(1);
-
-     } else if (sm->type == DETECT_ISDATAAT) {
-        {
-            SCLogDebug("inspecting isdataat");
-
-            DetectIsdataatData *id = (DetectIsdataatData *)sm->ctx;
-            if (id->flags & ISDATAAT_RELATIVE) {
-                if (det_ctx->payload_offset + id->dataat > payload_len) {
-                    SCLogDebug("det_ctx->payload_offset + id->dataat %"PRIu32" > %"PRIu32, det_ctx->payload_offset + id->dataat, payload_len);
-                    if (id->flags & ISDATAAT_NEGATED)
-                        goto match;
-                    SCReturnInt(0);
-                } else {
-                    SCLogDebug("relative isdataat match");
-                    if (id->flags & ISDATAAT_NEGATED)
-                        SCReturnInt(0);
-                    goto match;
-                }
-            } else {
-                if (id->dataat < payload_len) {
-                    SCLogDebug("absolute isdataat match");
-                    if (id->flags & ISDATAAT_NEGATED)
-                        SCReturnInt(0);
-                    goto match;
-                } else {
-                    SCLogDebug("absolute isdataat mismatch, id->isdataat %"PRIu32", payload_len %"PRIu32"", id->dataat,payload_len);
-                    if (id->flags & ISDATAAT_NEGATED)
-                        goto match;
-                    SCReturnInt(0);
-                }
-            }
-        }
-    } else if (sm->type == DETECT_PCRE) {
-        SCLogDebug("inspecting pcre");
-        DetectPcreData *pe = (DetectPcreData *)sm->ctx;
-        uint32_t prev_payload_offset = det_ctx->payload_offset;
-        uint32_t prev_offset = 0;
-        int r = 0;
-
-        det_ctx->pcre_match_start_offset = 0;
-        do {
-            r = DetectPcrePayloadMatch(det_ctx, s, sm, NULL, NULL,
-                                       payload, payload_len);
-
-            if (r == 0) {
-                det_ctx->discontinue_matching = 1;
-                SCReturnInt(0);
-            }
-
-            if (!(pe->flags & DETECT_PCRE_RELATIVE_NEXT)) {
-                SCLogDebug("no relative match coming up, so this is a match");
-                goto match;
-            }
-
-            /* save it, in case we need to do a pcre match once again */
-            prev_offset = det_ctx->pcre_match_start_offset;
-
-            /* see if the next payload keywords match. If not, we will
-             * search for another occurence of this pcre and see
-             * if the others match, until we run out of matches */
-            int r = DoInspectHttpServerBody(de_ctx, det_ctx, s, sm->next,
-                                            payload, payload_len);
-            if (r == 1) {
-                SCReturnInt(1);
-            }
-
-            if (det_ctx->discontinue_matching)
-                SCReturnInt(0);
-
-            det_ctx->payload_offset = prev_payload_offset;
-            det_ctx->pcre_match_start_offset = prev_offset;
-        } while (1);
-    } else if (sm->type == DETECT_BYTETEST) {
-        DetectBytetestData *btd = (DetectBytetestData *)sm->ctx;
-        int32_t offset = btd->offset;
-        uint64_t value = btd->value;
-        if (btd->flags & DETECT_BYTETEST_OFFSET_BE) {
-            offset = det_ctx->bj_values[offset];
-        }
-        if (btd->flags & DETECT_BYTETEST_VALUE_BE) {
-            value = det_ctx->bj_values[value];
-        }
-
-        if (DetectBytetestDoMatch(det_ctx,s,sm,payload,payload_len, btd->flags,
-                    offset, value) != 1) {
-            SCReturnInt(0);
-        }
-
-        goto match;
-    } else if (sm->type == DETECT_BYTEJUMP) {
-        DetectBytejumpData *bjd = (DetectBytejumpData *)sm->ctx;
-        int32_t offset = bjd->offset;
-
-        if (bjd->flags & DETECT_BYTEJUMP_OFFSET_BE) {
-            offset = det_ctx->bj_values[offset];
-        }
-
-        if (DetectBytejumpDoMatch(det_ctx,s,sm,payload,payload_len,
-                    bjd->flags, offset) != 1) {
-            SCReturnInt(0);
-        }
-
-        goto match;
-    } else if (sm->type == DETECT_BYTE_EXTRACT) {
-        DetectByteExtractData *bed = (DetectByteExtractData *)sm->ctx;
-
-        if (DetectByteExtractDoMatch(det_ctx, sm, s, payload,
-                    payload_len,
-                    &det_ctx->bj_values[bed->local_id],
-                    bed->endian) != 1) {
-            SCReturnInt(0);
-        }
-
-        goto match;
-    } else {
-        /* we should never get here, but bail out just in case */
-        SCLogDebug("sm->type %u", sm->type);
-#ifdef DEBUG
-        BUG_ON(1);
-#endif
-    }
-
-    SCReturnInt(0);
-
-match:
-    /* this sigmatch matched, inspect the next one. If it was the last,
-     * the payload portion of the signature matched. */
-    if (sm->next != NULL) {
-        int r = DoInspectHttpServerBody(de_ctx, det_ctx, s, sm->next, payload,
-                                        payload_len);
-        SCReturnInt(r);
-    } else {
-        SCReturnInt(1);
-    }
-}
+#define BODY_SCAN_WINDOW 4096
+#define BODY_MINIMAL_SIZE 32768
 
 /**
  * \brief Helps buffer response bodies for different transactions and stores them
@@ -391,19 +68,15 @@ match:
  * \param f         Pointer to the flow.
  * \param htp_state http state.
  *
- * \warning Make sure flow is locked.
+ * \warning Make sure flow is locked. Flow is modified, WRITE lock needed.
  */
 static void DetectEngineBufferHttpServerBodies(DetectEngineCtx *de_ctx,
-        DetectEngineThreadCtx *det_ctx, Flow *f, HtpState *htp_state)
+        DetectEngineThreadCtx *det_ctx, Flow *f, HtpState *htp_state, uint8_t flags)
 {
     int idx = 0;
     htp_tx_t *tx = NULL;
     int i = 0;
 
-    if (det_ctx->hsbd_buffers_list_len > 0) {
-        SCReturn;
-    }
-
     if (htp_state == NULL) {
         SCLogDebug("no HTTP state");
         goto end;
@@ -422,23 +95,22 @@ static void DetectEngineBufferHttpServerBodies(DetectEngineCtx *de_ctx,
 
     /* let's get the transaction count.  We need this to hold the server body
      * buffer for each transaction */
-    det_ctx->hsbd_buffers_list_len = list_size(htp_state->connp->conn->transactions) - tmp_idx;
+    size_t txs = list_size(htp_state->connp->conn->transactions) - tmp_idx;
     /* no transactions?!  cool.  get out of here */
-    if (det_ctx->hsbd_buffers_list_len == 0)
+    if (txs == 0) {
+        det_ctx->hsbd_buffers_list_len = 0;
         goto end;
+    } else if (txs > det_ctx->hsbd_buffers_list_len) {
+        det_ctx->hsbd = SCRealloc(det_ctx->hsbd, txs * sizeof(HttpReassembledBody));
+        if (det_ctx->hsbd == NULL) {
+            det_ctx->hsbd_buffers_list_len = 0;
+            goto end;
+        }
 
-    /* assign space to hold buffers.  Each per transaction */
-    det_ctx->hsbd_buffers = SCMalloc(det_ctx->hsbd_buffers_list_len * sizeof(uint8_t *));
-    if (det_ctx->hsbd_buffers == NULL) {
-        goto end;
+        memset(det_ctx->hsbd + det_ctx->hsbd_buffers_list_len, 0,
+                (txs - det_ctx->hsbd_buffers_list_len) * sizeof(HttpReassembledBody));
+        det_ctx->hsbd_buffers_list_len = txs;
     }
-    memset(det_ctx->hsbd_buffers, 0, det_ctx->hsbd_buffers_list_len * sizeof(uint8_t *));
-
-    det_ctx->hsbd_buffers_len = SCMalloc(det_ctx->hsbd_buffers_list_len * sizeof(uint32_t));
-    if (det_ctx->hsbd_buffers_len == NULL) {
-        goto end;
-    }
-    memset(det_ctx->hsbd_buffers_len, 0, det_ctx->hsbd_buffers_list_len * sizeof(uint32_t));
 
     idx = AppLayerTransactionGetInspectId(f);
     if (idx == -1) {
@@ -447,6 +119,9 @@ static void DetectEngineBufferHttpServerBodies(DetectEngineCtx *de_ctx,
 
     int size = (int)list_size(htp_state->connp->conn->transactions);
     for (; idx < size; idx++, i++) {
+        /* already set up */
+        if (det_ctx->hsbd[i].buffer_len > 0)
+            continue;
 
         tx = list_get(htp_state->connp->conn->transactions, idx);
         if (tx == NULL)
@@ -456,64 +131,93 @@ static void DetectEngineBufferHttpServerBodies(DetectEngineCtx *de_ctx,
         if (htud == NULL)
             continue;
 
-        HtpBodyChunk *cur = htud->response_body.first;
+        /* no new data */
+        if (htud->response_body.body_inspected == htud->response_body.content_len_so_far) {
+            continue;
+        }
 
-        if (htud->response_body.nchunks == 0) {
+        HtpBodyChunk *cur = htud->response_body.first;
+        if (cur == NULL) {
             SCLogDebug("No http chunks to inspect for this transacation");
             continue;
-        } else {
-            /* no chunks?!! move on to the next transaction */
-            if (cur == NULL) {
-                SCLogDebug("No http chunks to inspect");
-                continue;
-            }
+        }
 
-            /* in case of chunked transfer encoding, we don't have the length
-             * of the response body until we see a chunk with length 0.  This
-             * doesn't let us use the response body callback function to
-             * figure out the end of response body.  Instead we do it here.  If
-             * the length is 0, and we have already seen content, it indicates
-             * chunked transfer.  We also check if the parser has truly seen
-             * the last chunk by checking the progress state for the
-             * transaction.  If we are done parsing all the chunks, we would
-             * have it set to something other than TX_PROGRESS_REQ_BODY.
-             * Either ways we should be moving away from buffering in the end
-             * and running content validation on this buffer type of architecture
-             * to a stateful inspection, where we can inspect body chunks as and
-             * when they come */
-            if (htud->response_body.content_len == 0) {
-                if ((htud->response_body.content_len_so_far > 0) &&
+        /* in case of chunked transfer encoding, we don't have the length
+         * of the response body until we see a chunk with length 0.  This
+         * doesn't let us use the response body callback function to
+         * figure out the end of response body.  Instead we do it here.  If
+         * the length is 0, and we have already seen content, it indicates
+         * chunked transfer.  We also check if the parser has truly seen
+         * the last chunk by checking the progress state for the
+         * transaction.  If we are done parsing all the chunks, we would
+         * have it set to something other than TX_PROGRESS_REQ_BODY.
+         * Either ways we should be moving away from buffering in the end
+         * and running content validation on this buffer type of architecture
+         * to a stateful inspection, where we can inspect body chunks as and
+         * when they come */
+        if (htud->response_body.content_len == 0) {
+            if ((htud->response_body.content_len_so_far > 0) &&
                     tx->progress != TX_PROGRESS_REQ_BODY) {
-                    /* final length of the body */
-                    htud->flags |= HTP_BODY_COMPLETE;
+                /* final length of the body */
+                htud->tcflags |= HTP_RES_BODY_COMPLETE;
+            }
+        }
+
+        if (flags & STREAM_EOF) {
+            htud->tcflags |= HTP_RES_BODY_COMPLETE;
+        }
+
+        /* inspect the body if the transfer is complete or we have hit
+         * our body size limit */
+        if (htud->response_body.content_len_so_far < BODY_MINIMAL_SIZE &&
+                !(htud->tcflags & HTP_RES_BODY_COMPLETE)) {
+            SCLogDebug("we still haven't seen the entire response body.  "
+                    "Let's defer body inspection till we see the "
+                    "entire body.");
+            continue;
+        }
+
+        //SCLogInfo("now we inspect! %"PRIu64, htud->response_body.content_len_so_far);
+
+        int first = 1;
+        while (cur != NULL) {
+            /* see if we can filter out chunks */
+            if (htud->response_body.body_inspected > 0) {
+                if (cur->stream_offset < htud->response_body.body_inspected) {
+                    if (htud->response_body.body_inspected - cur->stream_offset > BODY_SCAN_WINDOW) {
+                        cur = cur->next;
+                        continue;
+                    } else {
+                        /* include this one */
+                    }
+                } else {
+                    /* include this one */
                 }
             }
 
-            /* inspect the body if the transfer is complete or we have hit
-             * our body size limit */
-            if (!(htud->flags & HTP_BODY_COMPLETE)) {
-                SCLogDebug("we still haven't seen the entire response body.  "
-                        "Let's defer body inspection till we see the "
-                        "entire body.");
-                continue;
+            if (first) {
+                det_ctx->hsbd[i].offset = cur->stream_offset;
+                first = 0;
             }
 
-            uint8_t *chunks_buffer = NULL;
-            int32_t chunks_buffer_len = 0;
-            while (cur != NULL) {
-                chunks_buffer_len += cur->len;
-                if ( (chunks_buffer = SCRealloc(chunks_buffer, chunks_buffer_len)) == NULL) {
+            /* see if we need to grow the buffer */
+            if (det_ctx->hsbd[i].buffer == NULL || det_ctx->hsbd[i].buffer_len + cur->len > det_ctx->hsbd[i].buffer_size) {
+                det_ctx->hsbd[i].buffer_size += cur->len * 2;
+
+                if ((det_ctx->hsbd[i].buffer = SCRealloc(det_ctx->hsbd[i].buffer, det_ctx->hsbd[i].buffer_size)) == NULL) {
                     goto end;
                 }
-
-                memcpy(chunks_buffer + chunks_buffer_len - cur->len, cur->data, cur->len);
-                cur = cur->next;
             }
-            /* store the buffers.  We will need it for further inspection */
-            det_ctx->hsbd_buffers[i] = chunks_buffer;
-            det_ctx->hsbd_buffers_len[i] = chunks_buffer_len;
+            memcpy(det_ctx->hsbd[i].buffer + det_ctx->hsbd[i].buffer_len, cur->data, cur->len);
+            det_ctx->hsbd[i].buffer_len += cur->len;
 
-        } /* else - if (htud->body.nchunks == 0) */
+            cur = cur->next;
+        }
+
+        /* update inspected tracker */
+        htud->response_body.body_inspected =
+            htud->response_body.last->stream_offset +
+            htud->response_body.last->len;
     } /* for (idx = AppLayerTransactionGetInspectId(f); .. */
 
 end:
@@ -521,22 +225,26 @@ end:
 }
 
 int DetectEngineRunHttpServerBodyMpm(DetectEngineCtx *de_ctx,
-        DetectEngineThreadCtx *det_ctx, Flow *f, HtpState *htp_state)
+                                     DetectEngineThreadCtx *det_ctx, Flow *f,
+                                     HtpState *htp_state, uint8_t flags)
 {
     int i;
     uint32_t cnt = 0;
 
-    /* bail before locking if we have nothing to do */
-    if (det_ctx->hsbd_buffers_list_len == 0) {
-        SCMutexLock(&f->m);
-        DetectEngineBufferHttpServerBodies(de_ctx, det_ctx, f, htp_state);
-        SCMutexUnlock(&f->m);
-    }
+    FLOWLOCK_WRLOCK(f);
+    DetectEngineBufferHttpServerBodies(de_ctx, det_ctx, f, htp_state, flags);
+    FLOWLOCK_UNLOCK(f);
 
-    for (i = 0; i < det_ctx->hsbd_buffers_list_len; i++) {
-        cnt += HttpServerBodyPatternSearch(det_ctx,
-                                           det_ctx->hsbd_buffers[i],
-                                           det_ctx->hsbd_buffers_len[i]);
+    if (det_ctx->hsbd != NULL && det_ctx->hsbd_buffers_list_len) {
+        for (i = 0; i < det_ctx->hsbd_buffers_list_len; i++) {
+            if (det_ctx->hsbd[i].buffer_len == 0)
+                continue;
+
+            cnt += HttpServerBodyPatternSearch(det_ctx,
+                    det_ctx->hsbd[i].buffer,
+                    det_ctx->hsbd[i].buffer_len,
+                    flags);
+        }
     }
 
     return cnt;
@@ -564,24 +272,30 @@ int DetectEngineInspectHttpServerBody(DetectEngineCtx *de_ctx,
     int r = 0;
     int i = 0;
 
-    /* bail before locking if we have nothing to do */
-    if (det_ctx->hsbd_buffers_list_len == 0) {
-        SCMutexLock(&f->m);
-        DetectEngineBufferHttpServerBodies(de_ctx, det_ctx, f, alstate);
-        SCMutexUnlock(&f->m);
-    }
+    FLOWLOCK_WRLOCK(f);
+    DetectEngineBufferHttpServerBodies(de_ctx, det_ctx, f, alstate, flags);
+    FLOWLOCK_UNLOCK(f);
 
-    for (i = 0; i < det_ctx->hsbd_buffers_list_len; i++) {
-        uint8_t *hsbd_buffer = det_ctx->hsbd_buffers[i];
-        uint32_t hsbd_buffer_len = det_ctx->hsbd_buffers_len[i];
+    if (det_ctx->hsbd != NULL && det_ctx->hsbd_buffers_list_len) {
+        for (i = 0; i < det_ctx->hsbd_buffers_list_len; i++) {
+            uint8_t *hsbd_buffer = det_ctx->hsbd[i].buffer;
+            uint32_t hsbd_buffer_len = det_ctx->hsbd[i].buffer_len;
 
-        if (hsbd_buffer == NULL)
-            continue;
+            if (hsbd_buffer == NULL || hsbd_buffer_len == 0)
+                continue;
 
-        r = DoInspectHttpServerBody(de_ctx, det_ctx, s, s->sm_lists[DETECT_SM_LIST_HSBDMATCH],
-                                    hsbd_buffer, hsbd_buffer_len);
-        if (r == 1) {
-            break;
+            det_ctx->buffer_offset = 0;
+            det_ctx->discontinue_matching = 0;
+            det_ctx->inspection_recursion_counter = 0;
+
+            r = DetectEngineContentInspection(de_ctx, det_ctx, s, s->sm_lists[DETECT_SM_LIST_HSBDMATCH],
+                    f,
+                    hsbd_buffer,
+                    hsbd_buffer_len,
+                    DETECT_ENGINE_CONTENT_INSPECTION_MODE_HSBD, NULL);
+            if (r == 1) {
+                break;
+            }
         }
     }
 
@@ -595,23 +309,12 @@ int DetectEngineInspectHttpServerBody(DetectEngineCtx *de_ctx,
  */
 void DetectEngineCleanHSBDBuffers(DetectEngineThreadCtx *det_ctx)
 {
-    if (det_ctx->hsbd_buffers_list_len != 0) {
-        int i;
+    int i;
+    if (det_ctx->hsbd != NULL && det_ctx->hsbd_buffers_list_len) {
         for (i = 0; i < det_ctx->hsbd_buffers_list_len; i++) {
-            if (det_ctx->hsbd_buffers[i] != NULL)
-                SCFree(det_ctx->hsbd_buffers[i]);
+            det_ctx->hsbd[i].buffer_len = 0;
         }
-        if (det_ctx->hsbd_buffers != NULL) {
-            SCFree(det_ctx->hsbd_buffers);
-            det_ctx->hsbd_buffers = NULL;
-        }
-        if (det_ctx->hsbd_buffers_len != NULL) {
-            SCFree(det_ctx->hsbd_buffers_len);
-            det_ctx->hsbd_buffers_len = NULL;
-        }
-        det_ctx->hsbd_buffers_list_len = 0;
     }
-
     return;
 }
 
@@ -2210,7 +1913,7 @@ static int DetectEngineHttpServerBodyTest14(void)
 
     de_ctx->sig_list = SigInit(de_ctx,"alert http any any -> any any "
                                "(msg:\"http server body test\"; "
-                               "pcre:/ab/S; "
+                               "pcre:/ab/Q; "
                                "content:\"ef\"; http_server_body; distance:2; "
                                "sid:1;)");
     if (de_ctx->sig_list == NULL)
@@ -2329,7 +2032,7 @@ static int DetectEngineHttpServerBodyTest15(void)
 
     de_ctx->sig_list = SigInit(de_ctx,"alert http any any -> any any "
                                "(msg:\"http server body test\"; "
-                               "pcre:/abc/S; "
+                               "pcre:/abc/Q; "
                                "content:!\"xyz\"; http_server_body; distance:0; within:3; "
                                "sid:1;)");
     if (de_ctx->sig_list == NULL)
