@@ -586,7 +586,7 @@ static IPOnlyCIDRItem *IPOnlyCIDRListParse2(char *s, int negate)
     int o_set = 0, n_set = 0, d_set = 0;
     int depth = 0;
     size_t size = strlen(s);
-    char address[1024] = "";
+    char address[8196] = "";
     char *rule_var_address = NULL;
     char *temp_rule_var_address = NULL;
     IPOnlyCIDRItem *head;
@@ -679,7 +679,7 @@ static IPOnlyCIDRItem *IPOnlyCIDRListParse2(char *s, int negate)
         } else if (depth == 0 && s[u] == '$') {
             d_set = 1;
         } else if (depth == 0 && u == size - 1) {
-            if (x == 1024) {
+            if (x == sizeof(address)) {
                 address[x - 1] = '\0';
             } else {
                 address[x] = '\0';
@@ -1068,9 +1068,9 @@ void IPOnlyMatchPacket(ThreadVars *tv,
                     }
                     if ( !(s->flags & SIG_FLAG_NOALERT)) {
                         if (s->action & ACTION_DROP)
-                            PacketAlertAppend(det_ctx, s, p, PACKET_ALERT_FLAG_DROP_FLOW, NULL);
+                            PacketAlertAppend(det_ctx, s, p, PACKET_ALERT_FLAG_DROP_FLOW);
                         else
-                            PacketAlertAppend(det_ctx, s, p, 0, NULL);
+                            PacketAlertAppend(det_ctx, s, p, 0);
                     }
                 }
             }
@@ -1515,6 +1515,10 @@ void IPOnlyAddSignature(DetectEngineCtx *de_ctx, DetectEngineIPOnlyCtx *io_ctx,
 
     /* enable the sig in the bitarray */
     io_ctx->sig_init_array[(s->num/8)] |= 1 << (s->num % 8);
+
+    /** no longer ref to this, it's in the table now */
+    s->CidrSrc = NULL;
+    s->CidrDst = NULL;
 }
 
 #ifdef UNITTESTS
