@@ -914,10 +914,8 @@ static void SigGroupHeadFreeSigArraysHash2(DetectEngineCtx *de_ctx,
             sgh->init->sig_size = 0;
         }
 
-        if (sgh->init != NULL) {
-            SigGroupHeadInitDataFree(sgh->init);
-            sgh->init = NULL;
-        }
+        SigGroupHeadInitDataFree(sgh->init);
+        sgh->init = NULL;
     }
 
     return;
@@ -1403,7 +1401,7 @@ int SigGroupHeadLoadUricontent(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
             continue;
 
         for ( ;sm != NULL; sm = sm->next) {
-            if (sm->type == DETECT_URICONTENT) {
+            if (sm->type == DETECT_CONTENT) {
                 co = (DetectContentData *)sm->ctx;
 
                 sgh->init->uri_content_array[co->id / 8] |= 1 << (co->id % 8);
@@ -1575,10 +1573,10 @@ int SigGroupHeadBuildMatchArray(DetectEngineCtx *de_ctx, SigGroupHead *sgh,
 }
 
 /**
- *  \brief Set the filestore_cnt in the sgh.
+ *  \brief Set the need md5 flag in the sgh.
  *
  *  \param de_ctx detection engine ctx for the signatures
- *  \param sgh sig group head to set the counter in
+ *  \param sgh sig group head to set the flag in
  */
 void SigGroupHeadSetFilemagicFlag(DetectEngineCtx *de_ctx, SigGroupHead *sgh) {
     Signature *s = NULL;
@@ -1601,6 +1599,34 @@ void SigGroupHeadSetFilemagicFlag(DetectEngineCtx *de_ctx, SigGroupHead *sgh) {
     return;
 }
 
+/**
+ *  \brief Set the need magic flag in the sgh.
+ *
+ *  \param de_ctx detection engine ctx for the signatures
+ *  \param sgh sig group head to set the flag in
+ */
+void SigGroupHeadSetFileMd5Flag(DetectEngineCtx *de_ctx, SigGroupHead *sgh) {
+    Signature *s = NULL;
+    uint32_t sig = 0;
+
+    if (sgh == NULL)
+        return;
+
+    for (sig = 0; sig < sgh->sig_cnt; sig++) {
+        s = sgh->match_array[sig];
+        if (s == NULL)
+            continue;
+
+        if (SignatureIsFileMd5Inspecting(s)) {
+            sgh->flags |= SIG_GROUP_HEAD_HAVEFILEMD5;
+            SCLogDebug("sgh %p has filemd5", sgh);
+            break;
+        }
+    }
+
+    return;
+}
+
 /**
  *  \brief Set the filestore_cnt in the sgh.
  *
