@@ -229,7 +229,7 @@ int DetectFlowbitSetup (DetectEngineCtx *de_ctx, Signature *s, char *rawstr)
     if (cd == NULL)
         goto error;
 
-    cd->idx = VariableNameGetIdx(fb_name,DETECT_FLOWBITS);
+    cd->idx = VariableNameGetIdx(de_ctx, fb_name, DETECT_FLOWBITS);
     cd->cmd = fb_cmd;
 
     SCLogDebug("idx %" PRIu32 ", cmd %s, name %s",
@@ -257,14 +257,14 @@ int DetectFlowbitSetup (DetectEngineCtx *de_ctx, Signature *s, char *rawstr)
         case DETECT_FLOWBITS_CMD_ISNOTSET:
         case DETECT_FLOWBITS_CMD_ISSET:
             /* checks, so packet list */
-            SigMatchAppendPacket(s, sm);
+            SigMatchAppendSMToList(s, sm, DETECT_SM_LIST_MATCH);
             break;
 
         case DETECT_FLOWBITS_CMD_SET:
         case DETECT_FLOWBITS_CMD_UNSET:
         case DETECT_FLOWBITS_CMD_TOGGLE:
             /* modifiers, only run when entire sig has matched */
-            SigMatchAppendPostMatch(s, sm);
+            SigMatchAppendSMToList(s, sm, DETECT_SM_LIST_POSTMATCH);
             break;
     }
 
@@ -609,7 +609,7 @@ static int FlowBitsTestSig04(void) {
 
     s = de_ctx->sig_list = SigInit(de_ctx,"alert ip any any -> any any (msg:\"isset option\"; flowbits:isset,fbt; content:\"GET \"; sid:1;)");
 
-    idx = VariableNameGetIdx("fbt",DETECT_FLOWBITS);
+    idx = VariableNameGetIdx(de_ctx, "fbt", DETECT_FLOWBITS);
 
     if (s == NULL || idx != 1) {
         goto end;
@@ -790,7 +790,7 @@ static int FlowBitsTestSig06(void) {
 
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
 
-    idx = VariableNameGetIdx("myflow",DETECT_FLOWBITS);
+    idx = VariableNameGetIdx(de_ctx, "myflow", DETECT_FLOWBITS);
 
     gv = p->flow->flowvar;
 
@@ -896,7 +896,7 @@ static int FlowBitsTestSig07(void) {
 
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
 
-    idx = VariableNameGetIdx("myflow",DETECT_FLOWBITS);
+    idx = VariableNameGetIdx(de_ctx, "myflow", DETECT_FLOWBITS);
 
     gv = p->flow->flowvar;
 
@@ -1005,7 +1005,7 @@ static int FlowBitsTestSig08(void) {
 
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
 
-    idx = VariableNameGetIdx("myflow",DETECT_FLOWBITS);
+    idx = VariableNameGetIdx(de_ctx, "myflow", DETECT_FLOWBITS);
 
     gv = p->flow->flowvar;
 
