@@ -35,6 +35,8 @@
 #include "suricata.h"
 #include "decode.h"
 
+#include "host.h"
+
 #include "detect.h"
 #include "detect-parse.h"
 
@@ -242,7 +244,8 @@ static int DetectThresholdSetup (DetectEngineCtx *de_ctx, Signature *s, char *ra
     SigMatch *tmpm = NULL;
 
     /* checks if there is a previous instance of detection_filter */
-    tmpm = SigMatchGetLastSM(s->sm_lists[DETECT_SM_LIST_MATCH], DETECT_DETECTION_FILTER);
+    tmpm = SigMatchGetLastSMFromLists(s, 2,
+                                      DETECT_DETECTION_FILTER, s->sm_lists[DETECT_SM_LIST_MATCH]);
     if (tmpm != NULL) {
         SCLogError(SC_ERR_INVALID_SIGNATURE, "\"detection_filter\" and \"threshold\" are not allowed in the same rule");
         SCReturnInt(-1);
@@ -259,7 +262,7 @@ static int DetectThresholdSetup (DetectEngineCtx *de_ctx, Signature *s, char *ra
     sm->type = DETECT_THRESHOLD;
     sm->ctx = (void *)de;
 
-    SigMatchAppendPacket(s, sm);
+    SigMatchAppendSMToList(s, sm, DETECT_SM_LIST_THRESHOLD);
 
     return 0;
 
@@ -400,6 +403,8 @@ static int DetectThresholdTestSig1(void) {
     int result = 0;
     int alerts = 0;
 
+    HostInitConfig(HOST_QUIET);
+
     memset(&th_v, 0, sizeof(th_v));
 
     p = UTHBuildPacketReal((uint8_t *)"A",1,IPPROTO_TCP, "1.1.1.1", "2.2.2.2", 1024, 80);
@@ -427,20 +432,44 @@ static int DetectThresholdTestSig1(void) {
 
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts = PacketAlertCheck(p, 1);
+    if (alerts != 1) {
+        printf("alerts %"PRIi32", expected 1: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 2) {
+        printf("alerts %"PRIi32", expected 2: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 3) {
+        printf("alerts %"PRIi32", expected 3: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 4) {
+        printf("alerts %"PRIi32", expected 4: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 5) {
+        printf("alerts %"PRIi32", expected 5: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 5) {
+        printf("alerts %"PRIi32", expected 5: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 5) {
+        printf("alerts %"PRIi32", expected 5: ", alerts);
+    }
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     alerts += PacketAlertCheck(p, 1);
+    if (alerts != 5) {
+        printf("alerts %"PRIi32", expected 5: ", alerts);
+    }
 
     if(alerts == 5)
         result = 1;
@@ -454,6 +483,8 @@ static int DetectThresholdTestSig1(void) {
     DetectEngineCtxFree(de_ctx);
 
     UTHFreePackets(&p, 1);
+
+    HostShutdown();
 end:
     return result;
 }
@@ -475,6 +506,8 @@ static int DetectThresholdTestSig2(void) {
     int result = 0;
     int alerts = 0;
 
+    HostInitConfig(HOST_QUIET);
+
     memset(&th_v, 0, sizeof(th_v));
 
     p = UTHBuildPacketReal((uint8_t *)"A",1,IPPROTO_TCP, "1.1.1.1", "2.2.2.2", 1024, 80);
@@ -529,6 +562,7 @@ cleanup:
 
 end:
     UTHFreePackets(&p, 1);
+    HostShutdown();
     return result;
 }
 
@@ -549,9 +583,9 @@ static int DetectThresholdTestSig3(void) {
     int result = 0;
     int alerts = 0;
     struct timeval ts;
-    DetectThresholdData *td = NULL;
     DetectThresholdEntry *lookup_tsh = NULL;
-    DetectThresholdEntry *ste = NULL;
+
+    HostInitConfig(HOST_QUIET);
 
     memset (&ts, 0, sizeof(struct timeval));
     TimeGet(&ts);
@@ -575,41 +609,25 @@ static int DetectThresholdTestSig3(void) {
     SigGroupBuild(de_ctx);
     DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
 
-    td = SigGetThresholdType(s,p);
-
-    /* setup the Entry we use to search our hash with */
-    ste = SCMalloc(sizeof(DetectThresholdEntry));
-    if (ste == NULL)
-        goto end;
-    memset(ste, 0x00, sizeof(ste));
-
-    if (PKT_IS_IPV4(p))
-        ste->ipv = 4;
-    else if (PKT_IS_IPV6(p))
-        ste->ipv = 6;
-
-    ste->sid = s->id;
-    ste->gid = s->gid;
-
-    if (td->track == TRACK_DST) {
-        COPY_ADDRESS(&p->dst, &ste->addr);
-    } else if (td->track == TRACK_SRC) {
-        COPY_ADDRESS(&p->src, &ste->addr);
-    }
-
-    ste->track = td->track;
-
     TimeGet(&p->ts);
 
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
 
-    lookup_tsh = (DetectThresholdEntry *)HashListTableLookup(de_ctx->ths_ctx.threshold_hash_table_dst, ste, sizeof(DetectThresholdEntry));
+    Host *host = HostLookupHostFromHash(&p->dst);
+    if (host == NULL) {
+        printf("host not found: ");
+        goto cleanup;
+    }
+
+    lookup_tsh = (DetectThresholdEntry *)host->threshold;
     if (lookup_tsh == NULL) {
+        HostRelease(host);
         printf("lookup_tsh is NULL: ");
         goto cleanup;
     }
+    HostRelease(host);
 
     TimeSetIncrementTime(200);
     TimeGet(&p->ts);
@@ -618,8 +636,21 @@ static int DetectThresholdTestSig3(void) {
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
     SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
 
-    if (lookup_tsh)
-        alerts = lookup_tsh->current_count;
+    host = HostLookupHostFromHash(&p->dst);
+    if (host == NULL) {
+        printf("host not found: ");
+        goto cleanup;
+    }
+    HostRelease(host);
+
+    lookup_tsh = (DetectThresholdEntry *)host->threshold;
+    if (lookup_tsh == NULL) {
+        HostRelease(host);
+        printf("lookup_tsh is NULL: ");
+        goto cleanup;
+    }
+
+    alerts = lookup_tsh->current_count;
 
     if (alerts == 3)
         result = 1;
@@ -636,6 +667,7 @@ cleanup:
     DetectEngineCtxFree(de_ctx);
 end:
     UTHFreePackets(&p, 1);
+    HostShutdown();
     return result;
 }
 
@@ -657,6 +689,8 @@ static int DetectThresholdTestSig4(void) {
     int alerts = 0;
     struct timeval ts;
 
+    HostInitConfig(HOST_QUIET);
+
     memset (&ts, 0, sizeof(struct timeval));
     TimeGet(&ts);
 
@@ -710,6 +744,7 @@ cleanup:
     DetectEngineCtxFree(de_ctx);
 end:
     UTHFreePackets(&p, 1);
+    HostShutdown();
     return result;
 }
 
@@ -730,6 +765,8 @@ static int DetectThresholdTestSig5(void) {
     int result = 0;
     int alerts = 0;
 
+    HostInitConfig(HOST_QUIET);
+
     memset(&th_v, 0, sizeof(th_v));
     p = UTHBuildPacketReal((uint8_t *)"A",1,IPPROTO_TCP, "1.1.1.1", "2.2.2.2", 1024, 80);
 
@@ -792,6 +829,7 @@ cleanup:
 
 end:
     UTHFreePackets(&p, 1);
+    HostShutdown();
     return result;
 }
 
@@ -803,6 +841,8 @@ static int DetectThresholdTestSig6Ticks(void) {
     int result = 0;
     int alerts = 0;
 
+    HostInitConfig(HOST_QUIET);
+
     memset(&th_v, 0, sizeof(th_v));
     p = UTHBuildPacketReal((uint8_t *)"A",1,IPPROTO_TCP, "1.1.1.1", "2.2.2.2", 1024, 80);
 
@@ -871,6 +911,7 @@ cleanup:
 
 end:
     UTHFreePackets(&p, 1);
+    HostShutdown();
     return result;
 }
 #endif /* UNITTESTS */
