@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2012 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -40,8 +40,15 @@
 #include "util-time.h"
 #include "util-debug.h"
 
+#include "util-hash-lookup3.h"
+
 #define FLOW_DEFAULT_FLOW_PRUNE 5
 
+SC_ATOMIC_EXTERN(unsigned int, flow_prune_idx);
+SC_ATOMIC_EXTERN(unsigned char, flow_flags);
+
+static Flow *FlowGetUsedFlow(void);
+
 #ifdef FLOW_DEBUG_STATS
 #define FLOW_DEBUG_STATS_PROTO_ALL      0
 #define FLOW_DEBUG_STATS_PROTO_TCP      1
@@ -134,6 +141,54 @@ void FlowHashDebugDeinit(void) {
 
 #endif /* FLOW_DEBUG_STATS */
 
+/** \brief compare two raw ipv6 addrs
+ *
+ *  \note we don't care about the real ipv6 ip's, this is just
+ *        to consistently fill the FlowHashKey6 struct, without all
+ *        the ntohl calls.
+ *
+ *  \warning do not use elsewhere unless you know what you're doing.
+ *           detect-engine-address-ipv6.c's AddressIPv6GtU32 is likely
+ *           what you are looking for.
+ */
+static inline int FlowHashRawAddressIPv6GtU32(uint32_t *a, uint32_t *b)
+{
+    int i;
+
+    for (i = 0; i < 4; i++) {
+        if (a[i] > b[i])
+            return 1;
+        if (a[i] < b[i])
+            break;
+    }
+
+    return 0;
+}
+
+typedef struct FlowHashKey4_ {
+    union {
+        struct {
+            uint32_t src, dst;
+            uint16_t sp, dp;
+            uint16_t proto; /**< u16 so proto and recur add up to u32 */
+            uint16_t recur; /**< u16 so proto and recur add up to u32 */
+        };
+        uint32_t u32[4];
+    };
+} FlowHashKey4;
+
+typedef struct FlowHashKey6_ {
+    union {
+        struct {
+            uint32_t src[4], dst[4];
+            uint16_t sp, dp;
+            uint16_t proto; /**< u16 so proto and recur add up to u32 */
+            uint16_t recur; /**< u16 so proto and recur add up to u32 */
+        };
+        uint32_t u32[10];
+    };
+} FlowHashKey6;
+
 /* calculate the hash key for this packet
  *
  * we're using:
@@ -147,53 +202,107 @@ void FlowHashDebugDeinit(void) {
  *
  *  For ICMP we only consider UNREACHABLE errors atm.
  */
-uint32_t FlowGetKey(Packet *p) {
-    FlowKey *k = (FlowKey *)p;
+static inline uint32_t FlowGetKey(Packet *p) {
     uint32_t key;
 
     if (p->ip4h != NULL) {
         if (p->tcph != NULL || p->udph != NULL) {
-            key = (flow_config.hash_rand + k->proto + k->sp + k->dp + \
-                    k->src.addr_data32[0] + k->dst.addr_data32[0] + \
-                    k->recursion_level) % flow_config.hash_size;
-/*
-            SCLogDebug("TCP/UCP key %"PRIu32, key);
+            FlowHashKey4 fhk;
+            if (p->src.addr_data32[0] > p->dst.addr_data32[0]) {
+                fhk.src = p->src.addr_data32[0];
+                fhk.dst = p->dst.addr_data32[0];
+            } else {
+                fhk.src = p->dst.addr_data32[0];
+                fhk.dst = p->src.addr_data32[0];
+            }
+            if (p->sp > p->dp) {
+                fhk.sp = p->sp;
+                fhk.dp = p->dp;
+            } else {
+                fhk.sp = p->dp;
+                fhk.dp = p->sp;
+            }
+            fhk.proto = (uint16_t)p->proto;
+            fhk.recur = (uint16_t)p->recursion_level;
+
+            uint32_t hash = hashword(fhk.u32, 4, flow_config.hash_rand);
+            key = hash % flow_config.hash_size;
 
-            SCLogDebug("proto %u, sp %u, dp %u, src %u, dst %u, reclvl %u",
-                    k->proto, k->sp, k->dp, k->src.addr_data32[0], k->dst.addr_data32[0],
-                    k->recursion_level);
-*/
         } else if (ICMPV4_DEST_UNREACH_IS_VALID(p)) {
-//            SCLogDebug("valid ICMPv4 DEST UNREACH error packet");
+            uint32_t psrc = IPV4_GET_RAW_IPSRC_U32(ICMPV4_GET_EMB_IPV4(p));
+            uint32_t pdst = IPV4_GET_RAW_IPDST_U32(ICMPV4_GET_EMB_IPV4(p));
+            FlowHashKey4 fhk;
+            if (psrc > pdst) {
+                fhk.src = psrc;
+                fhk.dst = pdst;
+            } else {
+                fhk.src = pdst;
+                fhk.dst = psrc;
+            }
+            if (p->icmpv4vars.emb_sport > p->icmpv4vars.emb_dport) {
+                fhk.sp = p->icmpv4vars.emb_sport;
+                fhk.dp = p->icmpv4vars.emb_dport;
+            } else {
+                fhk.sp = p->icmpv4vars.emb_dport;
+                fhk.dp = p->icmpv4vars.emb_sport;
+            }
+            fhk.proto = (uint16_t)ICMPV4_GET_EMB_PROTO(p);
+            fhk.recur = (uint16_t)p->recursion_level;
 
-            key = (flow_config.hash_rand + ICMPV4_GET_EMB_PROTO(p) +
-                    p->icmpv4vars.emb_sport + \
-                    p->icmpv4vars.emb_dport + \
-                    IPV4_GET_RAW_IPSRC_U32(ICMPV4_GET_EMB_IPV4(p)) + \
-                    IPV4_GET_RAW_IPDST_U32(ICMPV4_GET_EMB_IPV4(p)) + \
-                    k->recursion_level) % flow_config.hash_size;
-/*
-            SCLogDebug("ICMP DEST UNREACH key %"PRIu32, key);
+            uint32_t hash = hashword(fhk.u32, 4, flow_config.hash_rand);
+            key = hash % flow_config.hash_size;
 
-            SCLogDebug("proto %u, sp %u, dp %u, src %u, dst %u, reclvl %u",
-                    ICMPV4_GET_EMB_PROTO(p), p->icmpv4vars.emb_sport,
-                    p->icmpv4vars.emb_dport, IPV4_GET_RAW_IPSRC_U32(ICMPV4_GET_EMB_IPV4(p)),
-                    IPV4_GET_RAW_IPDST_U32(ICMPV4_GET_EMB_IPV4(p)), k->recursion_level);
-*/
         } else {
-            key = (flow_config.hash_rand + k->proto + \
-                    k->src.addr_data32[0] + k->dst.addr_data32[0] + \
-                    k->recursion_level) % flow_config.hash_size;
+            FlowHashKey4 fhk;
+            if (p->src.addr_data32[0] > p->dst.addr_data32[0]) {
+                fhk.src = p->src.addr_data32[0];
+                fhk.dst = p->dst.addr_data32[0];
+            } else {
+                fhk.src = p->dst.addr_data32[0];
+                fhk.dst = p->src.addr_data32[0];
+            }
+            fhk.sp = 0xfeed;
+            fhk.dp = 0xbeef;
+            fhk.proto = (uint16_t)p->proto;
+            fhk.recur = (uint16_t)p->recursion_level;
 
+            uint32_t hash = hashword(fhk.u32, 4, flow_config.hash_rand);
+            key = hash % flow_config.hash_size;
         }
-    } else if (p->ip6h != NULL)
-        key = (flow_config.hash_rand + k->proto + k->sp + k->dp + \
-               k->src.addr_data32[0] + k->src.addr_data32[1] + \
-               k->src.addr_data32[2] + k->src.addr_data32[3] + \
-               k->dst.addr_data32[0] + k->dst.addr_data32[1] + \
-               k->dst.addr_data32[2] + k->dst.addr_data32[3] + \
-               k->recursion_level) % flow_config.hash_size;
-    else
+    } else if (p->ip6h != NULL) {
+        FlowHashKey6 fhk;
+        if (FlowHashRawAddressIPv6GtU32(p->src.addr_data32, p->dst.addr_data32)) {
+            fhk.src[0] = p->src.addr_data32[0];
+            fhk.src[1] = p->src.addr_data32[1];
+            fhk.src[2] = p->src.addr_data32[2];
+            fhk.src[3] = p->src.addr_data32[3];
+            fhk.dst[0] = p->dst.addr_data32[0];
+            fhk.dst[1] = p->dst.addr_data32[1];
+            fhk.dst[2] = p->dst.addr_data32[2];
+            fhk.dst[3] = p->dst.addr_data32[3];
+        } else {
+            fhk.src[0] = p->dst.addr_data32[0];
+            fhk.src[1] = p->dst.addr_data32[1];
+            fhk.src[2] = p->dst.addr_data32[2];
+            fhk.src[3] = p->dst.addr_data32[3];
+            fhk.dst[0] = p->src.addr_data32[0];
+            fhk.dst[1] = p->src.addr_data32[1];
+            fhk.dst[2] = p->src.addr_data32[2];
+            fhk.dst[3] = p->src.addr_data32[3];
+        }
+        if (p->sp > p->dp) {
+            fhk.sp = p->sp;
+            fhk.dp = p->dp;
+        } else {
+            fhk.sp = p->dp;
+            fhk.dp = p->sp;
+        }
+        fhk.proto = (uint16_t)p->proto;
+        fhk.recur = (uint16_t)p->recursion_level;
+
+        uint32_t hash = hashword(fhk.u32, 10, flow_config.hash_rand);
+        key = hash % flow_config.hash_size;
+    } else
         key = 0;
 
     return key;
@@ -300,56 +409,33 @@ static Flow *FlowGetNew(Packet *p) {
         return NULL;
     }
 
-    /* no, so get a new one */
+    /* get a flow from the spare queue */
     f = FlowDequeue(&flow_spare_q);
     if (f == NULL) {
-        /* If we reached the max memcap, try to clean some flows:
-         * 1- first by normal timeouts
-         * 2- by emergency mode timeouts
-         * 3- by last time seen
-         */
-        if ((SC_ATOMIC_GET(flow_memuse) + sizeof(Flow)) > flow_config.memcap) {
-            uint32_t not_released = 0;
+        /* If we reached the max memcap, we get a used flow */
+        if (!(FLOW_CHECK_MEMCAP(sizeof(Flow)))) {
+            /* declare state of emergency */
+            if (!(SC_ATOMIC_GET(flow_flags) & FLOW_EMERGENCY)) {
+                SC_ATOMIC_OR(flow_flags, FLOW_EMERGENCY);
 
-            SCLogDebug("We need to prune some flows(1)");
-
-            /* Ok, then try to release flow_try_release flows */
-            not_released = FlowPruneFlowsCnt(&p->ts, flow_config.flow_try_release);
-            if (not_released == (uint32_t)flow_config.flow_try_release) {
-                /* This means that none of the flows was released, so try again
-                 * with more agressive timeout values (emergency mode) */
-
-                if ( !(flow_flags & FLOW_EMERGENCY)) {
-                    SCLogWarning(SC_WARN_FLOW_EMERGENCY, "Warning, engine "
-                            "running with FLOW_EMERGENCY bit set "
-                            "(ts.tv_sec: %"PRIuMAX", ts.tv_usec:%"PRIuMAX")",
-                            (uintmax_t)p->ts.tv_sec, (uintmax_t)p->ts.tv_usec);
-                    flow_flags |= FLOW_EMERGENCY; /* XXX mutex this */
-                    FlowWakeupFlowManagerThread();
-                }
-                SCLogDebug("We need to prune some flows with emerg bit (2)");
-
-                not_released = FlowPruneFlowsCnt(&p->ts, FLOW_DEFAULT_FLOW_PRUNE);
-                if (not_released == (uint32_t)flow_config.flow_try_release) {
-                    /* Here the engine is on a real stress situation
-                     * Try to kill the last time seen "flow_try_release" flows
-                     * directly, ignoring timeouts */
-                    SCLogDebug("We need to KILL some flows (3)");
-                    not_released = FlowKillFlowsCnt(FLOW_DEFAULT_FLOW_PRUNE);
-                    if (not_released == (uint32_t)flow_config.flow_try_release) {
-                        return NULL;
-                    }
-                }
+                /* under high load, waking up the flow mgr each time leads
+                 * to high cpu usage. Flows are not timed out much faster if
+                 * we check a 1000 times a second. */
+                FlowWakeupFlowManagerThread();
             }
-        }
 
-        /* now see if we can alloc a new flow */
-        f = FlowAlloc();
-        if (f == NULL) {
-            return NULL;
-        }
+            f = FlowGetUsedFlow();
 
-        /* flow is initialized but *unlocked* */
+            /* freed a flow, but it's unlocked */
+        } else {
+            /* now see if we can alloc a new flow */
+            f = FlowAlloc();
+            if (f == NULL) {
+                return NULL;
+            }
+
+            /* flow is initialized but *unlocked* */
+        }
     } else {
         /* flow has been recycled before it went into the spare queue */
 
@@ -358,7 +444,7 @@ static Flow *FlowGetNew(Packet *p) {
 
     FlowIncrUsecnt(f);
 
-    SCMutexLock(&f->m);
+    FLOWLOCK_WRLOCK(f);
     return f;
 }
 
@@ -383,37 +469,36 @@ Flow *FlowGetFlowFromHash (Packet *p)
     uint32_t key = FlowGetKey(p);
     /* get our hash bucket and lock it */
     FlowBucket *fb = &flow_hash[key];
-    SCSpinLock(&fb->s);
+    FBLOCK_LOCK(fb);
 
-    SCLogDebug("fb %p fb->f %p", fb, fb->f);
+    SCLogDebug("fb %p fb->head %p", fb, fb->head);
 
     FlowHashCountIncr;
 
     /* see if the bucket already has a flow */
-    if (fb->f == NULL) {
-        f = fb->f = FlowGetNew(p);
+    if (fb->head == NULL) {
+        f = FlowGetNew(p);
         if (f == NULL) {
-            SCSpinUnlock(&fb->s);
+            FBLOCK_UNLOCK(fb);
             FlowHashCountUpdate;
             return NULL;
         }
 
         /* flow is locked */
+        fb->head = f;
+        fb->tail = f;
 
         /* got one, now lock, initialize and return */
         FlowInit(f,p);
-        f->flags |= FLOW_NEW_LIST;
         f->fb = fb;
 
-        FlowEnqueue(&flow_new_q[f->protomap], f);
-
-        SCSpinUnlock(&fb->s);
+        FBLOCK_UNLOCK(fb);
         FlowHashCountUpdate;
         return f;
     }
 
     /* ok, we have a flow in the bucket. Let's find out if it is our flow */
-    f = fb->f;
+    f = fb->head;
 
     /* see if this is the flow we are looking for */
     if (FlowCompare(f, p) == 0) {
@@ -428,10 +513,11 @@ Flow *FlowGetFlowFromHash (Packet *p)
             if (f == NULL) {
                 f = pf->hnext = FlowGetNew(p);
                 if (f == NULL) {
-                    SCSpinUnlock(&fb->s);
+                    FBLOCK_UNLOCK(fb);
                     FlowHashCountUpdate;
                     return NULL;
                 }
+                fb->tail = f;
 
                 /* flow is locked */
 
@@ -439,13 +525,9 @@ Flow *FlowGetFlowFromHash (Packet *p)
 
                 /* initialize and return */
                 FlowInit(f,p);
-
-                f->flags |= FLOW_NEW_LIST;
                 f->fb = fb;
 
-                FlowEnqueue(&flow_new_q[f->protomap], f);
-
-                SCSpinUnlock(&fb->s);
+                FBLOCK_UNLOCK(fb);
                 FlowHashCountUpdate;
                 return f;
             }
@@ -453,18 +535,25 @@ Flow *FlowGetFlowFromHash (Packet *p)
             if (FlowCompare(f, p) != 0) {
                 /* we found our flow, lets put it on top of the
                  * hash list -- this rewards active flows */
-                if (f->hnext) f->hnext->hprev = f->hprev;
-                if (f->hprev) f->hprev->hnext = f->hnext;
+                if (f->hnext) {
+                    f->hnext->hprev = f->hprev;
+                }
+                if (f->hprev) {
+                    f->hprev->hnext = f->hnext;
+                }
+                if (f == fb->tail) {
+                    fb->tail = f->hprev;
+                }
 
-                f->hnext = fb->f;
+                f->hnext = fb->head;
                 f->hprev = NULL;
-                fb->f->hprev = f;
-                fb->f = f;
+                fb->head->hprev = f;
+                fb->head = f;
 
                 /* found our flow, lock & return */
                 FlowIncrUsecnt(f);
-                SCMutexLock(&f->m);
-                SCSpinUnlock(&fb->s);
+                FLOWLOCK_WRLOCK(f);
+                FBLOCK_UNLOCK(fb);
                 FlowHashCountUpdate;
                 return f;
             }
@@ -473,9 +562,80 @@ Flow *FlowGetFlowFromHash (Packet *p)
 
     /* lock & return */
     FlowIncrUsecnt(f);
-    SCMutexLock(&f->m);
-    SCSpinUnlock(&fb->s);
+    FLOWLOCK_WRLOCK(f);
+    FBLOCK_UNLOCK(fb);
     FlowHashCountUpdate;
     return f;
 }
 
+/** \internal
+ *  \brief Get a flow from the hash directly.
+ *
+ *  Called in conditions where the spare queue is empty and memcap is reached.
+ *
+ *  Walks the hash until a flow can be freed. Timeouts are disregarded, use_cnt
+ *  is adhered to. "flow_prune_idx" atomic int makes sure we don't start at the
+ *  top each time since that would clear the top of the hash leading to longer
+ *  and longer search times under high pressure (observed).
+ *
+ *  \retval f flow or NULL
+ */
+static Flow *FlowGetUsedFlow(void) {
+    uint32_t idx = SC_ATOMIC_GET(flow_prune_idx) % flow_config.hash_size;
+    uint32_t cnt = flow_config.hash_size;
+
+    while (cnt--) {
+        if (idx++ >= flow_config.hash_size)
+            idx = 0;
+
+        FlowBucket *fb = &flow_hash[idx];
+        if (fb == NULL)
+            continue;
+
+        if (FBLOCK_TRYLOCK(fb) != 0)
+            continue;
+
+        Flow *f = fb->tail;
+        if (f == NULL) {
+            FBLOCK_UNLOCK(fb);
+            continue;
+        }
+
+        if (FLOWLOCK_TRYWRLOCK(f) != 0) {
+            FBLOCK_UNLOCK(fb);
+            continue;
+        }
+
+        /** never prune a flow that is used by a packet or stream msg
+         *  we are currently processing in one of the threads */
+        if (SC_ATOMIC_GET(f->use_cnt) > 0) {
+            FBLOCK_UNLOCK(fb);
+            FLOWLOCK_UNLOCK(f);
+            continue;
+        }
+
+        /* remove from the hash */
+        if (f->hprev != NULL)
+            f->hprev->hnext = f->hnext;
+        if (f->hnext != NULL)
+            f->hnext->hprev = f->hprev;
+        if (fb->head == f)
+            fb->head = f->hnext;
+        if (fb->tail == f)
+            fb->tail = f->hprev;
+
+        f->hnext = NULL;
+        f->hprev = NULL;
+        f->fb = NULL;
+        FBLOCK_UNLOCK(fb);
+
+        FlowClearMemory (f, f->protomap);
+
+        FLOWLOCK_UNLOCK(f);
+
+        (void) SC_ATOMIC_ADD(flow_prune_idx, (flow_config.hash_size - cnt));
+        return f;
+    }
+
+    return NULL;
+}
