@@ -188,7 +188,7 @@ static void LogDropLogDeInitCtx(OutputCtx *output_ctx)
 static void CreateTimeString (const struct timeval *ts, char *str, size_t size) {
     time_t time = ts->tv_sec;
     struct tm local_tm;
-    struct tm *t = (struct tm *)localtime_r(&time, &local_tm);
+    struct tm *t = (struct tm *)SCLocalTime(time, &local_tm);
 
     snprintf(str, size, "%02d/%02d/%02d-%02d:%02d:%02d.%06u",
             t->tm_mon + 1, t->tm_mday, t->tm_year + 1900, t->tm_hour,
