@@ -19,7 +19,7 @@
  * \file
  *
  * \author Endace Technology Limited.
- * \author Jason MacLulich <jason.maclulich@eendace.com>
+ * \author Jason MacLulich <jason.maclulich@endace.com>
  *
  * Support for reading ERF records from a DAG card.
  *
@@ -56,6 +56,7 @@ void TmModuleDecodeErfDagRegister (void) {
     tmm_modules[TMM_DECODEERFDAG].ThreadDeinit = NULL;
     tmm_modules[TMM_DECODEERFDAG].RegisterTests = NULL;
     tmm_modules[TMM_DECODEERFDAG].cap_flags = 0;
+    tmm_modules[TMM_DECODEERFDAG].flags = TM_FLAG_DECODE_TM;
 }
 
 TmEcode NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
@@ -68,8 +69,6 @@ TmEcode NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
 
 #else /* Implied we do have DAG support */
 
-#define DAG_MAX_READ_PKTS 256
-
 #include "source-erf-dag.h"
 #include <dagapi.h>
 
@@ -78,35 +77,31 @@ extern uint8_t suricata_ctl_flags;
 
 typedef struct ErfDagThreadVars_ {
     ThreadVars *tv;
+    TmSlot *slot;
+
     int dagfd;
     int dagstream;
     char dagname[DAGNAME_BUFSIZE];
-    uint32_t dag_max_read_packets;
 
     struct timeval maxwait, poll;   /* Could possibly be made static */
 
     uint32_t pkts;
     uint64_t bytes;
 
-    /* Track current location in the DAG stream input buffer
+    /* Current location in the DAG stream input buffer.
      */
-    uint8_t* top;                   /* We track top as well so we don't have to
-                                       call dag_advance_stream again if there
-                                       are still pkts to process.
-
-                                       JNM: Currently not used.
-                                     */
-    uint8_t* btm;
+    uint8_t *top;
+    uint8_t *btm;
 
 } ErfDagThreadVars;
 
-TmEcode ReceiveErfDag(ThreadVars *, Packet *, void *, PacketQueue *, PacketQueue *);
+static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
+    uint32_t *pkts_read);
+static inline TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec);
+TmEcode ReceiveErfDagLoop(ThreadVars *, void *data, void *slot);
 TmEcode ReceiveErfDagThreadInit(ThreadVars *, void *, void **);
 void ReceiveErfDagThreadExitStats(ThreadVars *, void *);
 TmEcode ReceiveErfDagThreadDeinit(ThreadVars *, void *);
-TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, Packet *p, uint8_t* top,
-                             PacketQueue *postpq, uint32_t *pkts_read);
-TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec, Packet *p);
 
 TmEcode DecodeErfDagThreadInit(ThreadVars *, void *, void **);
 TmEcode DecodeErfDag(ThreadVars *, Packet *, void *, PacketQueue *, PacketQueue *);
@@ -120,12 +115,14 @@ TmModuleReceiveErfDagRegister(void)
 {
     tmm_modules[TMM_RECEIVEERFDAG].name = "ReceiveErfDag";
     tmm_modules[TMM_RECEIVEERFDAG].ThreadInit = ReceiveErfDagThreadInit;
-    tmm_modules[TMM_RECEIVEERFDAG].Func = ReceiveErfDag;
+    tmm_modules[TMM_RECEIVEERFDAG].Func = NULL;
+    tmm_modules[TMM_RECEIVEERFDAG].PktAcqLoop = ReceiveErfDagLoop;
     tmm_modules[TMM_RECEIVEERFDAG].ThreadExitPrintStats =
         ReceiveErfDagThreadExitStats;
     tmm_modules[TMM_RECEIVEERFDAG].ThreadDeinit = NULL;
     tmm_modules[TMM_RECEIVEERFDAG].RegisterTests = NULL;
     tmm_modules[TMM_RECEIVEERFDAG].cap_flags = 0;
+    tmm_modules[TMM_RECEIVEERFDAG].flags = TM_FLAG_RECEIVE_TM;
 }
 
 /**
@@ -141,6 +138,7 @@ TmModuleDecodeErfDagRegister(void)
     tmm_modules[TMM_DECODEERFDAG].ThreadDeinit = NULL;
     tmm_modules[TMM_DECODEERFDAG].RegisterTests = NULL;
     tmm_modules[TMM_DECODEERFDAG].cap_flags = 0;
+    tmm_modules[TMM_DECODEERFDAG].flags = TM_FLAG_DECODE_TM;
 }
 
 /**
@@ -179,13 +177,6 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
 
     memset(ewtn, 0, sizeof(*ewtn));
 
-    /*  Use max_pending_packets as our maximum number of packets read
-     from the DAG buffer.
-     */
-    ewtn->dag_max_read_packets = (DAG_MAX_READ_PKTS < max_pending_packets) ?
-        DAG_MAX_READ_PKTS : max_pending_packets;
-
-
     /* dag_parse_name will return a DAG device name and stream number
      * to open for this thread.
      */
@@ -274,9 +265,9 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
      * Initialise DAG Polling parameters.
      */
     timerclear(&ewtn->maxwait);
-    ewtn->maxwait.tv_usec = 100 * 1000; /* 100ms timeout */
+    ewtn->maxwait.tv_usec = 20 * 1000; /* 20ms timeout */
     timerclear(&ewtn->poll);
-    ewtn->poll.tv_usec = 10 * 1000; /* 10ms poll interval */
+    ewtn->poll.tv_usec = 1 * 1000; /* 1ms poll interval */
 
     /* 32kB minimum data to return -- we still restrict the number of
      * pkts that are processed to a maximum of dag_max_read_packets.
@@ -300,191 +291,148 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
 }
 
 /**
- * \brief   Thread entry function for reading ERF records from a DAG card.
+ * \brief Receives packets from a DAG interface.
  *
- *          Reads a new ERF record the DAG input buffer and copies it to
- *          an internal Suricata packet buffer -- similar to the way the
- *          pcap packet handler works.
+ * \param tv pointer to ThreadVars
+ * \param data pointer to ErfDagThreadVars
+ * \param slot slot containing task information
  *
- *          We create new packet structures using PacketGetFromQueueOrAlloc
- *          for each packet between the top and btm pointers except for
- *          the first packet for which a Packet buffer is provided
- *          from the packetpool.
- *
- *          We always read up to dag_max_read_packets ERF packets from the
- *          DAG buffer, but we might read less. This differs from the
- *          ReceivePcap handler -- it will only read pkts up to a maximum
- *          of either the packetpool count or the pcap_max_read_packets.
- *
- * \param   tv pointer to ThreadVars
- * \param   p data pointer
- * \param   data
- * \param   pq pointer to the PacketQueue (not used here)
- * \param   postpq
- * \retval  TM_ECODE_FAILED on failure and TM_ECODE_OK on success.
- * \note    We also use the packetpool hack first used in the source-pcap
- *          handler so we don't keep producing packets without any dying.
- *          This implies that if we are in this situation we run the risk
- *          of dropping packets at the interface.
+ * \retval TM_ECODE_OK on success
+ * \retval TM_ECODE_FAILED on failure
  */
-TmEcode
-ReceiveErfDag(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
-               PacketQueue *postpq)
+TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
 {
-    SCEnter();
-
-    uint16_t packet_q_len = 0;
+    ErfDagThreadVars *dtv = (ErfDagThreadVars *)data;
+    TmSlot *s = (TmSlot *)slot;
+    dtv->slot = s->slot_next;
     uint32_t diff = 0;
     int      err;
     uint8_t  *top = NULL;
     uint32_t pkts_read = 0;
 
-    assert(p);
-    assert(pq);
-    assert(postpq);
+    SCEnter();
 
-    ErfDagThreadVars *ewtn = (ErfDagThreadVars *)data;
-
-    /* NOTE/JNM: Hack copied from source-pcap.c
-     *
-     * Make sure we have at least one packet in the packet pool, to
-     * prevent us from alloc'ing packets at line rate
-     */
-    while (packet_q_len == 0) {
-        packet_q_len = PacketPoolSize();
-        if (packet_q_len == 0) {
-            PacketPoolWait();
-        }
-    }
-
-    if (postpq == NULL) {
-        ewtn->dag_max_read_packets = 1;
-    }
-
-    while(pkts_read == 0)
+    while (1)
     {
-	    if (suricata_ctl_flags != 0) {
-            break;
+        if (suricata_ctl_flags & SURICATA_STOP ||
+            suricata_ctl_flags & SURICATA_KILL) {
+            SCReturnInt(TM_ECODE_FAILED);
         }
 
-        /* NOTE/JNM: This might not work well if we start restricting the
-	     * number of ERF records processed per call to a small number as
-	     * the over head required here could exceed the time it takes to
-	     * process a small number of ERF records.
-	     *
-	     * XXX/JNM: Possibly process the DAG stream buffer first if there
-	     * are ERF packets or else call dag_advance_stream and then process
-	     * the DAG stream buffer.
-	     */
-	    top = dag_advance_stream(ewtn->dagfd, ewtn->dagstream, &(ewtn->btm));
-
-	    if (NULL == top)
-	    {
-	        if((ewtn->dagstream & 0x1) && (errno == EAGAIN)) {
-	            usleep(10 * 1000);
-	            ewtn->btm = ewtn->top;
+        top = dag_advance_stream(dtv->dagfd, dtv->dagstream, &(dtv->btm));
+        if (top == NULL) {
+            if (errno == EAGAIN) {
+                if (dtv->dagstream & 0x1) {
+                    usleep(10 * 1000);
+                    dtv->btm = dtv->top;
+                }
                 continue;
-	        }
-	        else {
-	            SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
-	                       "Failed to read from stream: %d, DAG: %s when using dag_advance_stream",
-	                       ewtn->dagstream, ewtn->dagname);
-	            SCReturnInt(TM_ECODE_FAILED);
-	        }
-	    }
+            }
+            else {
+                SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
+                    "Failed to read from stream: %d, DAG: %s when using dag_advance_stream",
+                    dtv->dagstream, dtv->dagname);
+                SCReturnInt(TM_ECODE_FAILED);
+            }
+        }
 
-	    diff = top - ewtn->btm;
-	    if (diff == 0)
-	    {
-	        continue;
-	    }
+        diff = top - dtv->btm;
+        if (diff == 0) {
+            continue;
+        }
 
-	    assert(diff >= dag_record_size);
+        assert(diff >= dag_record_size);
 
-	    err = ProcessErfDagRecords(ewtn, p, top, postpq, &pkts_read);
+        err = ProcessErfDagRecords(dtv, top, &pkts_read);
 
         if (err == TM_ECODE_FAILED) {
-             SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
-                   "Failed to read from stream: %d, DAG: %s",
-                   ewtn->dagstream, ewtn->dagname);
-            ReceiveErfDagCloseStream(ewtn->dagfd, ewtn->dagstream);
+            SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
+                "Failed to read from stream: %d, DAG: %s",
+                dtv->dagstream, dtv->dagname);
+            ReceiveErfDagCloseStream(dtv->dagfd, dtv->dagstream);
             SCReturnInt(err);
         }
-    }
 
-    SCLogDebug("Read %d records from stream: %d, DAG: %s",
-        pkts_read, ewtn->dagstream, ewtn->dagname);
+        SCLogDebug("Read %d records from stream: %d, DAG: %s",
+            pkts_read, dtv->dagstream, dtv->dagname);
+    }
 
     if (suricata_ctl_flags != 0) {
         SCReturnInt(TM_ECODE_FAILED);
     }
 
-    SCReturnInt(err);
+    SCReturnInt(TM_ECODE_OK);
 }
 
-TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn,
-                             Packet *p,
-                             uint8_t* top,
-                             PacketQueue *postpq,
-                             uint32_t *pkts_read)
+/**
+ * \brief Process a chunk of records read from a DAG interface.
+ *
+ * This function takes a pointer to buffer read from the DAG interface
+ * and processes it individual records.
+ */
+static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
+    uint32_t *pkts_read)
 {
     SCEnter();
 
-    int     err = 0;
-    dag_record_t* dr = NULL;
-    char    *prec = NULL;
-    int     rlen;
+    int err = 0;
+    dag_record_t *dr = NULL;
+    char *prec = NULL;
+    int rlen;
+    char hdr_type = 0;
+    int processed = 0;
+    int packet_q_len = 0;
 
     *pkts_read = 0;
 
-    while(((top-(ewtn->btm))>=dag_record_size) &&
-          ((*pkts_read)<(ewtn->dag_max_read_packets)))
-    {
-        prec = (char*)ewtn->btm;
+    while (((top - ewtn->btm) >= dag_record_size) &&
+        ((processed + dag_record_size) < 4*1024*1024)) {
+
+        /* Make sure we have at least one packet in the packet pool,
+         * to prevent us from alloc'ing packets at line rate. */
+        do {
+            packet_q_len = PacketPoolSize();
+            if (unlikely(packet_q_len == 0)) {
+                PacketPoolWait();
+            }
+        } while (packet_q_len == 0);
+
+        prec = (char *)ewtn->btm;
         dr = (dag_record_t*)prec;
-
         rlen = ntohs(dr->rlen);
-
-        if (rlen == 20) {
-            rlen = 28;
-            SCLogWarning(SC_WARN_ERF_DAG_REC_LEN_CHANGED,
-                "Warning, adjusted the length of ERF from 20 to 28 on stream: %d, DAG: %s",
-                ewtn->dagstream, ewtn->dagname);
-        }
+        hdr_type = dr->type;
 
         /* If we don't have enough data to finsih processing this ERF record
          * return and maybe next time we will.
          */
-        if ((top-(ewtn->btm)) < rlen)
+        if ((top - ewtn->btm) < rlen)
             SCReturnInt(TM_ECODE_OK);
 
-        p = p ? p : PacketGetFromQueueOrAlloc();
+        ewtn->btm += rlen;
+        processed += rlen;
 
-        if (p == NULL) {
-            SCLogError(SC_ERR_MEM_ALLOC,
-                       "Failed to allocate a Packet on stream: %d, DAG: %s",
-                       ewtn->dagstream, ewtn->dagname);
+        /* Only support ethernet at this time. */
+        switch (hdr_type & 0x7f) {
+        case TYPE_PAD:
+            /* Skip. */
+            continue;
+        case TYPE_ETH:
+        case TYPE_DSM_COLOR_ETH:
+        case TYPE_COLOR_ETH:
+        case TYPE_COLOR_HASH_ETH:
+            break;
+        default:
+            SCLogError(SC_ERR_UNIMPLEMENTED,
+                "Processing of DAG record type: %d not implemented.", dr->type);
             SCReturnInt(TM_ECODE_FAILED);
         }
 
-        err = ProcessErfDagRecord(ewtn, prec, p);
-
-        if (err != TM_ECODE_OK)
-            SCReturnInt(err);
-
-        ewtn->btm += rlen;
-
-        /* XXX/JNM: Hack to get around the fact that the first Packet from
-         * Suricata is added explicitly by the Slot code and shouldn't go
-         * onto the post queue -- else it is added twice to the next queue.
-         */
-        if (*pkts_read) {
-            PacketEnqueue(postpq, p);
+        err = ProcessErfDagRecord(ewtn, prec);
+        if (err != TM_ECODE_OK) {
+            SCReturnInt(TM_ECODE_FAILED);
         }
 
         (*pkts_read)++;
-
-        p = NULL;
     }
 
     SCReturnInt(TM_ECODE_OK);
@@ -495,32 +443,49 @@ TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn,
  * \param   prec pointer to a DAG record.
  * \param
  */
-TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec, Packet *p)
+static inline TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec)
 {
     SCEnter();
 
     int wlen = 0;
+    int rlen = 0;
+    int hdr_num = 0;
+    char hdr_type = 0;
     dag_record_t  *dr = (dag_record_t*)prec;
     erf_payload_t *pload;
+    Packet *p;
 
-    assert(prec);
-    assert(p);
+    hdr_type = dr->type;
+    wlen = ntohs(dr->wlen);
+    rlen = ntohs(dr->rlen);
 
-    if (p == NULL) SCReturnInt(TM_ECODE_OK);
-
-    /* Only support ethernet at this time. */
-    if (dr->type != TYPE_ETH &&
-	    dr->type != TYPE_DSM_COLOR_ETH &&
-	    dr->type != TYPE_COLOR_ETH &&
-	    dr->type != TYPE_COLOR_HASH_ETH) {
-        SCLogError(SC_ERR_UNIMPLEMENTED,
-                   "Processing of DAG record type: %d not implemented.", dr->type);
-        SCReturnInt(TM_ECODE_FAILED);
+    /* count extension headers */
+    while (hdr_type & 0x80) {
+        if (rlen < (dag_record_size + (hdr_num * 8))) {
+            SCLogError(SC_ERR_UNIMPLEMENTED,
+                "Insufficient captured packet length.");
+            SCReturnInt(TM_ECODE_FAILED);
+        }
+        hdr_type = prec[(dag_record_size + (hdr_num * 8))];
+        hdr_num++;
     }
 
-    wlen = ntohs(dr->wlen);
+    /* Check that the whole frame was captured */
+    if (rlen < (dag_record_size + (8 * hdr_num) + 2 + wlen)) {
+        SCLogInfo("Incomplete frame captured.");
+        SCReturnInt(TM_ECODE_OK);
+    }
 
-    pload = &(dr->rec);
+    /* skip over extension headers */
+    pload = (erf_payload_t *)(prec + dag_record_size + (8 * hdr_num));
+
+    p = PacketGetFromQueueOrAlloc();
+    if (p == NULL) {
+        SCLogError(SC_ERR_MEM_ALLOC,
+            "Failed to allocate a Packet on stream: %d, DAG: %s",
+            ewtn->dagstream, ewtn->dagname);
+        SCReturnInt(TM_ECODE_FAILED);
+    }
 
     SET_PKT_LEN(p, wlen - 4);   /* Trim the FCS... */
     p->datalink = LINKTYPE_ETHERNET;
@@ -528,10 +493,10 @@ TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec, Packet *p)
     /* Take into account for link type Ethernet ETH frame starts
      * after ther ERF header + pad.
      */
-    PacketCopyData(p, pload->eth.dst, GET_PKT_LEN(p));
-
-    SCLogDebug("pktlen: %" PRIu32 " (pkt %02x, pkt data %02x)",
-               GET_PKT_LEN(p), *p, *GET_PKT_DATA(p));
+    if (unlikely(PacketCopyData(p, pload->eth.dst, GET_PKT_LEN(p)))) {
+        TmqhOutputPacketpool(ewtn->tv, p);
+        SCReturnInt(TM_ECODE_FAILED);
+    }
 
     /* Convert ERF time to timeval - from libpcap. */
     uint64_t ts = dr->ts;
@@ -547,6 +512,11 @@ TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec, Packet *p)
     ewtn->pkts++;
     ewtn->bytes += wlen;
 
+    if (TmThreadsSlotProcessPkt(ewtn->tv, ewtn->slot, p) != TM_ECODE_OK) {
+        TmqhOutputPacketpool(ewtn->tv, p);
+        SCReturnInt(TM_ECODE_FAILED);
+    }
+
     SCReturnInt(TM_ECODE_OK);
 }
 
