@@ -155,8 +155,10 @@
             #define __WORDSIZE LONG_BIT
         #endif
     #endif
+#endif
 
 /** Windows does not define __WORDSIZE, but it uses __X86__ */
+#ifndef __WORDSIZE
 	#if defined(__X86__) || defined(_X86_)
 		#define __WORDSIZE 32
 	#else
@@ -184,6 +186,9 @@ typedef enum PacketProfileDetectId_ {
     PROF_DETECT_MPM_HMD,
     PROF_DETECT_MPM_HCD,
     PROF_DETECT_MPM_HRUD,
+    PROF_DETECT_MPM_HSMD,
+    PROF_DETECT_MPM_HSCD,
+    PROF_DETECT_MPM_HUAD,
     PROF_DETECT_IPONLY,
     PROF_DETECT_RULES,
     PROF_DETECT_STATEFUL,
