@@ -99,6 +99,8 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_REASSEMBLY);
         CASE_CODE (SC_ERR_POOL_INIT);
         CASE_CODE (SC_ERR_UNIMPLEMENTED);
+        CASE_CODE (SC_ERR_ADDRESS_ENGINE_GENERIC);
+        CASE_CODE (SC_ERR_PORT_ENGINE_GENERIC);
         CASE_CODE (SC_ERR_FAST_LOG_GENERIC);
         CASE_CODE (SC_ERR_IPONLY_RADIX);
         CASE_CODE (SC_ERR_DEBUG_LOG_GENERIC);
@@ -188,6 +190,12 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_ERF_DAG_STREAM_SET_FAILED);
         CASE_CODE (SC_ERR_ERF_DAG_STREAM_READ_FAILED);
         CASE_CODE (SC_WARN_ERF_DAG_REC_LEN_CHANGED);
+        CASE_CODE (SC_ERR_NAPATECH_OPEN_FAILED);
+        CASE_CODE (SC_ERR_NAPATECH_FEED_OPEN_FAILED);
+        CASE_CODE (SC_ERR_NAPATECH_FEED_START_FAILED);
+        CASE_CODE (SC_ERR_NAPATECH_FEED_NEXT_FAILED);
+        CASE_CODE (SC_ERR_NAPATECH_REQUIRED);
+        CASE_CODE (SC_ERR_NAPATECH_NOSUPPORT);
         CASE_CODE (SC_WARN_COMPATIBILITY);
         CASE_CODE (SC_ERR_DCERPC);
         CASE_CODE (SC_ERR_AHO_CORASICK);
@@ -215,7 +223,14 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_SOCKET);
         CASE_CODE (SC_ERR_PCAP_TRANSLATE);
         CASE_CODE (SC_WARN_OUTDATED_LIBHTP);
-
+        CASE_CODE (SC_WARN_DEPRECATED);
+        CASE_CODE (SC_WARN_PROFILE);
+        CASE_CODE (SC_ERR_FLOW_INIT);
+        CASE_CODE (SC_ERR_HOST_INIT);
+        CASE_CODE (SC_ERR_MEM_BUFFER_API);
+        CASE_CODE (SC_ERR_INVALID_MD5);
+        CASE_CODE (SC_ERR_NO_MD5_SUPPORT);
+        CASE_CODE (SC_ERR_EVENT_ENGINE);
         default:
             return "UNKNOWN_ERROR";
     }
