@@ -117,6 +117,7 @@ typedef enum {
     SC_ERR_DAEMON,
     SC_ERR_UNIMPLEMENTED,
     SC_ERR_ADDRESS_ENGINE_GENERIC,
+    SC_ERR_PORT_ENGINE_GENERIC,
     SC_ERR_IPONLY_RADIX,
     SC_ERR_FAST_LOG_GENERIC,
     SC_ERR_DEBUG_LOG_GENERIC,
@@ -198,6 +199,12 @@ typedef enum {
     SC_WARN_ERF_DAG_REC_LEN_CHANGED,
     SC_ERR_DAG_REQUIRED,
     SC_ERR_DAG_NOSUPPORT,           /**< no ERF/DAG support compiled in */
+    SC_ERR_NAPATECH_OPEN_FAILED,
+    SC_ERR_NAPATECH_FEED_OPEN_FAILED,
+    SC_ERR_NAPATECH_FEED_START_FAILED,
+    SC_ERR_NAPATECH_FEED_NEXT_FAILED,
+    SC_ERR_NAPATECH_REQUIRED,
+    SC_ERR_NAPATECH_NOSUPPORT,
     SC_ERR_FATAL,
     SC_ERR_DCERPC,
     SC_ERR_DETECT_PREPARE,          /**< preparing the detection engine failed */
@@ -230,6 +237,14 @@ typedef enum {
     SC_ERR_SOCKET,
     SC_ERR_PCAP_TRANSLATE,          /* failed to translate ip to dev */
     SC_WARN_OUTDATED_LIBHTP,
+    SC_WARN_DEPRECATED,
+    SC_WARN_PROFILE,
+    SC_ERR_FLOW_INIT,
+    SC_ERR_HOST_INIT,
+    SC_ERR_MEM_BUFFER_API,
+    SC_ERR_INVALID_MD5,
+    SC_ERR_NO_MD5_SUPPORT,
+    SC_ERR_EVENT_ENGINE,
 } SCError;
 
 const char *SCErrorToString(SCError);
